/**
 * Copyright (c) 2011, Andre Steingress
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 * 1.) Redistributions of source code must retain the above copyright notice, this list of conditions and the following
 * disclaimer.
 * 2.) Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided with the distribution.
 * 3.) Neither the name of Andre Steingress nor the names of its contributors may be used to endorse or
 * promote products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.gcontracts.annotations.meta;

import java.lang.annotation.Documented;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;

/**
 * <p>Specifies an annotation closure which will be part of a method's pre- or
 * postcondition.</p>
 *
 * For example:
 * <pre>
 *   &#064;Retention(RetentionPolicy.RUNTIME)
 *   &#064;Target(ElementType.PARAMETER)
 *
 *   &#064;Precondition
 *   &#064;AnnotationContract({ it != null })
 *   public @interface NotNull {}
 * </pre>
 *
 * <p>In the example above, {@code @NotNull} can be applied on parameters and will be
 * automatically injected into the precondition of the current method:</p>
 *
 * <pre>
 * def some_method(@NotNull def myParam)
 * </pre>
 *
 * <p>It is possible to specify multiple meta-annotations, currently only {@link Postcondition}
 * and {@link Precondition} are supported. </p>
 *
 * @see org.gcontracts.common.spi.AnnotationProcessor
 * @see Precondition
 * @see Postcondition
 *
 * @author ast
 */

@Documented
@Retention(RetentionPolicy.RUNTIME)
public @interface AnnotationContract {
    Class value();
}
